function forbidNewIndex(tab,key,value)
	error("Trying to assign value (of type ["..type(value).."]) for non-existing key/entry/member ["..key.."] in tab [TODO]"..debug.traceback() );
end

function forbidUnitializedIndex(tab,key)
	error("Trying to access non-existing key ["..key.."] in tab [TODO]"..debug.traceback() );
end



-- Should be called from C only
--_CalledFromC
function killBattery(id)
	log_info("===> Lua function [killBattery] called from C++ ["..id.."]");
	--log_info("ID TYPE "..type(id) );
	TaskManager:removeTask( tostring(id) );
end





TaskManager = {

	ReturnCodes = {
					["WaitingForInput"] = 1,
					["CreateNewTask"] = 2,
					["DeadCoroutine"] = 3,
				};
	tasks = {};
	free_ids = {};
}



--newindex
--local TaskManager_mt = {};



local tasks_metatable = {
	__index = function (tab, key)
				--local key_type = type(key);

				--assert(key_type == "number" or key_type == "string", "key_type should be either a string or a number");

				log_info("__index function: Creating new array for task of id ["..key.."] (type "..type(key)..")");
				tab[key] = Task.createNewTask();
				return tab[key];
			end,

	--__newindex = forbidNewIndex,

}

setmetatable(TaskManager.tasks,tasks_metatable)

function TaskManager:tostring()

	return "\n======== LUA MANAGER: =========== \nTasks table size:"..#self.tasks.."\n=====================================\n";
end





-- weird
TaskManager.__tostring = TaskManager:tostring();
-- k => weak table => won't get garbage collected
TaskManager.__mode  = 'k';
-- should prevent modification of metatable
TaskManager.__metatable = false

--TaskManager.__call

-- Retrieve task from id
TaskManager.__index = function(tab,key)
	return tab.tasks[key];
end




--TaskManager.__newindex = forbidNewIndex;

setmetatable(TaskManager,TaskManager)


--print(TaskManager);



function TaskManager.setParent(childId, parentId)

	assert(parentId, "parameter of type "..type(parentId).."\n"..debug.traceback() );
	log_info("Setting parent of task ["..childId.."] : new parent ["..parentId.."]")



	local task = self.tasks[childId];
	task["parent_task_id"] = parentId;

	table.insert(self.tasks[parentId]["subtask_ids"], childId );

end



-- on pourrait utiliser setfenv ici
function TaskManager:removeTask(id)

    log_info("Removing task ["..id.."] ");


	local task = self.tasks[id];


	--local parentId = task["parent_task_id"] ;

	-- if task to remove was blocking its parent
	if task["blocking"] then
		Task.unblock( self.tasks [ task["parent_task_id"] ] );
	end



	-- remove all children of this task
	for index,subtask_id in ipairs(task["subtask_ids"]) do

		log_info("Removing child ["..subtask_id.."] of task ["..id.."] ");

		if self.tasks[subtask_id]["parent_task_id"] == id then
			self:removeTask(subtask_id);
		else
			log_warning("Abnormal behavior : Wanted to erase wrong child !")
		end
	end


	-- to disable task update
    --task["action"]  = false;

	-- add id to available ids
	table.insert(self.free_ids, id);
end





-- function followed by its initialisation argument activator,
-- parentId
function TaskManager:addTask( fn, ... )

    -- should check task is a task
    --assert( type(fn) == "function", "\"task\" should be a function ! Current type ["..type(fn).."]\n"..debug.traceback(), fn);
    assert( type(comment) == "string", "task should have a description");

    -- generateId();
	if not id then
		-- gets and pop last element from cache
		id = table.remove(self.free_ids);
		if not id then
			id = #self.tasks + 1;
		end
	end


    log_info("Creating new Task with id ["..id.."]");

    -- if will create the task 'cos of the metatable
    --
	local task = self.tasks[id] ;
    Task.setup(task,fn,...)

-- function




    log_info("Task ["..id.."] created");

	return id;
end
--
--
--
--function TaskManager:resume_coroutine( id, co, ...)
--
--	local args =  ...;
--	local errorfree, code, arg1, arg2;
--	while true do
--				--unpack(optionals_args_table)
--				--
--				--local errorfree, getCurrentId = coroutine.resume( co, ... )
--				-- selon le code on va faire different trucs
--				-- stopper cette routine ci et en démarrer une autre
--				errorfree, code, arg1, arg2 = coroutine.resume( co, args )
--
--
--				--wait()
--
--
--				--if no error
--				-- and newtask
--				if not errorfree then
--					--log_error("Could not start coroutine ! Error ["..value.."]\nArguments: ");
--					error("Could not start coroutine !\nError [",code,"]\nArguments: ",debug.traceback(co));
--					--return false;
--
--				elseif code == TaskManager.ReturnCodes.WaitingForInput then
--					log_info("Waiting for input");
--					local reply = -1;
--					while reply < 0 do
--						reply = MainState:getUserReply();
--						--StopMainThread
--						coroutine.yield()
--					end
--					args = reply;
--					log_info("Answer received", args);
--				elseif code == TaskManager.ReturnCodes.CreateNewTask then
--					local new_task_id	= arg1
--
--					log_info("Creating child task id "..id.." ===")
--
--					-- set parent of the new task
--					local newTask 		= self.tasks[new_task_id];
--					local parentTask 	= self.tasks[ id ];
--
--					self:setParent(new_task_id,id);
--
--
--					if blocking then
--						newTask["blocking"] 	= true;
--						parentTask["blocked"] 	= true;
--						return true
--					end
--
--				-- if just yielded
--				else
--					return true;
--				end
--
--	end
--
--end



-- TODO la on pourrait ptet merger resume_coroutine et cette fct
-- reflechit a tt
function TaskManager:update()

    --elapsedTime

    --Returns the running coroutine plus a boolean, true when the running coroutine is the main one
    log_info ( "Running coroutine " , coroutine.running() )
    -- should be an assert,even !

	--print("Luamanager update");
	--ipairs iterates over indexes
    for id,task in pairs(self.tasks) do

        while true do

            -- arg1 is optional
            local code,arg1 = Task.process(task);

            -- according to code
            if code == TaskManager.ReturnCodes.DeadCoroutine then
                self:removeTask(id);
            elseif code == TaskManager.ReturnCodes.CreateNewTask then
				self:addTask( );

        end
--		local co = task["action"];


		--if not action or if blocked then continue to next task
--		if task["blocked"] or not co then
--			-- skip this item
--			--print("id ["..id.."] skipped");
--		else
--
--
--
--			local coStatus = coroutine.status(co);
--
--			-- if first launch of coroutine
--			--print("Treating id ["..id.."]" );
--			if not task["launched"] then
--
--				log_info("Starting coroutine ["..id.."]");
--				self:resume_coroutine(id,co, table.unpack(task["args"]) );
--				task["launched"] = true;
--				-- TODO
--				--task["args"] = nil
--
--				--print("launched:",task["launched"] );
--			-- if not dead yet
--			elseif coStatus ~= "dead" then
--
--					-- resume coroutine
--					self:resume_coroutine(id,co);
--
--			--dead coroutine
--			else
--				log_info("Coroutine ["..id.."] dead");
--				--task
--				self:removeTask(id);
--			end
--		end
	end

end

