/**
* \file bullet.h
* Fichier dans dossier "engine"

* bullet
**/
#ifndef BC_IStaticBattery_HPP
#define BC_IStaticBattery_HPP

//#include "../utils/IClock.hpp"
#include "../utils/TClock.hpp"
#include <irrlicht.h>
#include <array>
#include <list>
#include <luabind/lua_include.hpp>
//#include <luabind/operator.hpp>
#include "../def.hpp"




namespace NParameters
{
enum EId
{
Spinning = 0,
Angle,
Speed,
Damage,
Range,
KillAllChildren,
OrientationSpeed,
Life,
LateralSpeed,
ForwardSpeed,
Count       // doit rester en dernier
};
}

class CMissile;
class CBullet;
class CStaticBattery;
class IFleet;


float normalizeAngle(const float& angle);

void getVectorFromAngle(const float& angle,TVector3df& vectorToRotate);


/// Describe a bullet
// TODO faire deriver de luabind::wrap_base
class IStaticBattery
{

public:
    irr::scene::IAnimatedMeshSceneNode* _node;



    float _range;   //!< Radius of the disk

    int _offscreenCounter;
    IFleet* _fleet;

    float _rotationSpeed;

    int _bonus;

    /** \brief
     *
     */
    typedef std::list<IStaticBattery*> TMissileList;
    TMissileList _trackingMissiles;

    /** \brief Useful to compute accurate collisions
     *
     * \param
     *
     */

    TVector3df _previousPos;      //!< Entity's 3D position, useful to compute collisions

    int _damageInflictedOnCollision;

protected:

    /** \brief If it is aiming
    *
    */
    IStaticBattery* _target;
//    void setWithinScreen(const bool& value);
    int _life;

//    bool _withinScreen;     // if out of screen add a timer, if recome into screen, reset timer

    //TTimer _timer;


public:

    /** \brief
     *
     * \param  Fleet the entity belongs to
     * \param
     * \return
     *
     */
//    IStaticBattery( IFleet* , const irr::core::vector3df& pos, const irr::core::vector3df& forwardVec);
    IStaticBattery();
    virtual ~IStaticBattery();


    /** \brief
     *
     * \param
     * \param
     * \return
     *
     */

    void setTarget(IStaticBattery* );


    TVector3df computeLastFrameSpeedVector() const { return (this->getPosition() - _previousPos); };

    // Ptet virtual
    TVector3df predictPositionIn(const TTimeUnit&) const ;

    int stillOutOfScreen(const bool&);

    const int& getOffscreenCounter() const { return _offscreenCounter;};

    void setMesh(irr::scene::IAnimatedMesh* mesh);

    /** \brief
     *
     * \param
     * \param
     * \return
     *
     */

    IFleet* getFleet() { return _fleet; };


    void setFleet(IFleet* );

    /** \brief
     *
     * \param
     * \param
     * \return
     *
     */
    bool checkCollision(IStaticBattery*);


    /** \brief
     *
     * \param
     * \param
     * \return
     *
     */
    void addTrackingBattery(IStaticBattery*);



    void removeTrackingBattery(IStaticBattery*);



    virtual const EEntityType::EId getType() const = 0 ;



    //bool operator==(const CStaticBattery&) const;

    void setup(
//               const char* model,
               irr::scene::IAnimatedMesh*,
               const float& range,
               const int& life,
               const int& damage,
               const int& bonus
               );



    // Range setter/getter
    //void setRange(const float& range) { _range = range;};
    /** \brief Batteries are considered as disks. This is the radius of this disk
    **/
    virtual float getRange() const { return _range;};


    /** \brief Remaining Life of this battery. <0 battery is considered dead
    **/
    //void setLife(const int& life) { _life = life;};
    virtual int getRemainingLife() const { return _life;};


    /** \brief Bonus earned in case you destroy this battery
    **/
    //void setBonus(const int& bonus) { _bonus = bonus;};
    virtual float getBonus() const { return _bonus;};


    /** \brief
     *
     * \param
     * \param
     * \return
     *
     */
    void setRotationSpeed(const float& rotationSpeed)
    {
        _rotationSpeed = rotationSpeed;
    };


    //const bool& getCollision() const { return _collision;};

    /** \brief What this battery would inflict on collision
    **/
    const int& getInflictingDamage() const { return _damageInflictedOnCollision;};


    void setPosition(const float& x,const float& y,const float& z)
    {
        setPosition( TVector3df (x,y,z));
    }

    void setPosition(const TVector3df& );


    void exportPositionToLua(lua_State* L) const;



    const TVector3df&  getPosition() const {
        return _node->getPosition();
    };

    void translate(const float&,const float&,const float&);

    /**
    Every function about rotation
    **/
    const TVector3df&  getRotation() const { return _node->getRotation(); };
    TVector3df computeAnglesToFacePosition( const TVector3df& pos) const;
    void setRotation( const TVector3df& rotation );
    void updateRotation();

    //bool onShoot(const o3::CShooting& id);


    /** \brief Add damage to this entity
     */
    void addDamage(const int& damage);


    /**
    virtual functions
    **/
    //virtual void updateRotation();
    //virtual void updatePosition();
    virtual void onTimeoutDisparition();

    virtual void onDeath();



    // TODO a deplacer dans CPlayer ?
    CBullet* shootBullet(const char* model,const float& deviation);


    //void reset();


    /**
    * \param Time elapsed
    * \return True if should die
    **/
    virtual bool update(const TTimer::TimeType&);


    /** \brief
     *
     * \param
     * \param
     * \return
     *
     */

//    bool outOfScreenTimeout() const;


    /** \brief
     *
     * \param
     * \param
     * \return
     *
     */
    virtual void kill();

    virtual void onGettingCached();
    /**
    **/
//    virtual void onVisibleAreaEntrance() = 0;
//    virtual void onVisibleAreaExit() = 0;

//    IStaticBattery* _next;
//    IStaticBattery* _previous;

    // std::forward_list<>
    /// Single linked list

//    virtual void onCollision(IStaticBattery*);

//    friend std::ostream& operator<<(std::ostream&, const CStaticBattery& );

protected:


private:


};



#endif
