#include "CDialogStateLua.hpp"
#include <cmath>
#include <irrlicht.h>
#include "../config.hpp"
#include "../game_engine.hpp"
#include "../states/CErrorState.hpp"
#include "../IdList.hpp"
#include <fusion/scripting/CLuaThread.hpp>
#include <fusion/scripting/CLuaVM.hpp>
#include <fusion/maths/macros.hpp>
#include <fusion/core/string.hpp>
//#include "../CSpeech.hpp"
//#include "../CSpeaker.hpp"
#include "../binds/binds.hpp"

using namespace irr;
using namespace fus;


int continuationFunc(lua_State *L)
{
    _LOG_WARNING << "CONTINUATION function called !!";
}


CDialogStateLua::CDialogStateLua(lua_State* L,const std::string& pathToLuaFile) :
    IDialogState( "Lua dialog")
    , PathToLuaFile( pathToLuaFile )
    , Thread(0)
    , AnswerType(NAnswerType::Any)
{

//TODO
//    ADD_REQUIRED(Thread);

    // cstor
    //MainLuaContext
    //Threads are subject to garbage collection, like any Lua object.
    Thread = lua_newthread(L);
}

CDialogStateLua::~CDialogStateLua()
{

}


void
CDialogStateLua::onAnswer(const irr::s32& answerId)
{
    // TODO , selon le type de question - est-ce qu'on demandait un prénom ? un chiffre etc ?

//    lua_pushstring( Thread->getVM(), answerId );
//    lua_pushinteger( Thread, 2 );

//    WaitingForUserInput = false;
    clearAnswers();
}


bool CDialogStateLua::load(CGameEngine* engine,irr::scene::ISceneManager* smgr,irr::video::ITexture* screenshot)
{
    //! load dialog state
    if (! IDialogState::load(engine, smgr,screenshot) ) return false;


    // else bind every dialog related func to new thread
//    Thread = lua_newthread(MainLuaContext);

    // Need to bind everything dialogState related
    try {

        // On peut passer un nom de namespace
        //luabind::module( this->getVM(), "logger" )[
        luabind::module( this->Thread )[
            //scope
            bindDialogFunctionsToLua()
        ];


        luabind::object g;
        g = luabind::globals( Thread );
        g["Dialog"] = this;



        //luaL_dofile( gThread , "dialog2.lua");
        // "t" for text files and "b" for "binary"
        int res = luaL_loadfilex( Thread , PathToLuaFile.c_str() ,"bt");
        if( res != LUA_OK )
        {
            stackDump(Thread);
            return false;
        }

    }
    //catch(luabind::error& e)
    catch(...)
    {
        _LOG_ERROR << "exception catched";
        return false;
    }


	return true;
}



int
CDialogStateLua::getReply()
{
 //   return 0;

    _INFO << "getReply called";
//    WaitingForUserInput = true;
    waitForUserInput();

    int ctx = 0;
    lua_getctx( Thread, &ctx);
    AnswerType = NAnswerType::Integer;
//    lua_pushinteger( Thread, NAnswerType::Integer);

//int lua_yieldk  (lua_State *L, int nresults, int ctx, lua_CFunction k);
//When a C function calls lua_yieldk in that way, the running coroutine suspends its execution, and the call to lua_resume that started
//this coroutine returns.
//The parameter nresults is the number of values from the stack that are passed as results to lua_resume.
//    return lua_yieldk( Thread,0,ctx,continuationFunc);
    return lua_yield( Thread,0);
}



void
CDialogStateLua::onUpdate(const TTimeUnit& elapsedTime)
{


    //
    IDialogState::onUpdate( elapsedTime );


    if(waitingForUserInput() )
    {
//        cout << "Press a number to answer question" << endl;
//
//        cin >> gUserReply;
//        //gUserReply
//        lua_pushnumber(thread,2);
//        nargs = 1;
//        waitingForAnswer = false;
//        _INFO << "Waiting for userInput" ;
    }
    else
    {
        int result = 0;
        lua_State* cor = 0;

        if(AnswerType == NAnswerType::Any)
        {
//
        }
        else {
            cor = Thread;
//            result = 1;
//            lua_pushinteger( Thread, 3);
//            if(AnswerType == NAnswerType::Integer)
//            {
//
//            }
            //switch(AnswerType)
        }



        int nargs = 0;
        _INFO << "resuming with narg = " << nargs
              << " and nresults = ";

        //    int lua_resume (lua_State *L, lua_State *from, int narg);
/**
Starts and resumes a coroutine in a given thread.

 To start a coroutine, you push onto the thread stack the main function plus any arguments;
 then you call lua_resume, with narg being the number of arguments. This call returns when the coroutine suspends or finishes its execution.
  When it returns, the stack contains all values passed to lua_yield, or all values returned by the body function.
   lua_resume returns LUA_YIELD if the coroutine yields, LUA_OK if the coroutine finishes its execution without errors, or
    an error code in case of errors (see lua_pcall).

 In case of errors, the stack is not unwound, so you can use the debug API over it. The error message is on the top of the stack.

 To resume a coroutine, you put on its stack only the values to be passed as results from yield, and then call lua_resume.

 The parameter from represents the coroutine that is resuming L. If there is no such coroutine, this parameter can be NULL.**/
        result = lua_resume( Thread, cor, 0);

    // tant que script n'est pas fini
//    while( result != LUA_OK)
//    {

        if (result == LUA_YIELD)
        {
            _INFO << "Yielding" ;
//            getUserReply(thread);

            nargs = 1;
        }
        else if (result == LUA_OK)
        {
            _INFO << "script finished";

            // Should pop this state
            // Maybe we should pop this state after a blank screen ?
            engine().popState();
        }
        else
        {
            _LOG_ERROR << "we got errors" ;
            engine().popupMessage(L"Lua error",L"an error happened during dialog script");
            stackDump( Thread);
            // we should clear stack
        }


        AnswerType = NAnswerType::Any;
    }

    // TODO resume le
}
